#
#  Copyright 1999-2006 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 1.3

####################################################################
#                                                                  #
#     The following routine is the main routine for setup          #
#     and modification of motor scans.                             #
#                                                                  #
####################################################################

proc mxgui_list_scan_setup { record_list scan_name scan_type \
				scan_record old_record_exists } {

# Create the toplevel widget for the scan setup screen.

	set window .motor_scan_setup

	set status [ catch { toplevel $window } ]

# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $window
		focus $window
		return
	}

# Hide the window for now.

	wm withdraw $window
	wm title $window "List scan setup"

#
# Display the scan name and type at the top of the window.
#

	frame $window.name_frame
	pack $window.name_frame -padx 5 -pady 5 -expand yes -fill x

	label $window.name_frame.scan_name1 -text "Scan name:"
	pack $window.name_frame.scan_name1 -side left

	label $window.name_frame.scan_name2 -text $scan_name \
					-font $MxGui::monospaced_font
	pack $window.name_frame.scan_name2 -side left

	label $window.name_frame.pad -text " " -width 10
	pack $window.name_frame.pad -side left -expand yes -fill x

	label $window.name_frame.scan_type1 -text "Scan type:"
	pack $window.name_frame.scan_type1 -side left

	switch -- $scan_type {
		file_list_scan { set scan_label "File list scan" }
		list_scan { set scan_label "File list scan" }
		default    { set scan_label "Unrecognized" }
	}

	label $window.name_frame.scan_type2 -text $scan_label \
					-font $MxGui::monospaced_font
	pack $window.name_frame.scan_type2 -side left

	frame $window.separator0 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator0 -expand yes -fill both

#
# Initialize the display of the number of times to perform the scan.
#

	mxgui_initialize_num_scans_field $window $record_list \
				$scan_record $old_record_exists

	frame $window.separator1 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator1 -expand yes -fill both

#
# Initialize the motor display or, for an input scan, the number of steps
# in the scan.
#

	mxgui_list_initialize_motor_display \
			$window $record_list $scan_record \
			$old_record_exists

	frame $window.separator2 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator2 -expand yes -fill both

#
# Setup the rest of the standard scan display fields.  The last argument
# to mxgui_initialize_common_scan_display is set to 1 so that the measurement
# parameters are included in the standard scan display fields.
#
	set show_measurement_parameters 1

	mxgui_initialize_standard_scan_display $window $record_list \
				$scan_record $scan_type $old_record_exists \
				$show_measurement_parameters

#
# Display the OK and Cancel buttons.
#

	frame $window.b
	pack $window.b

	set buttonbox $window.b.buttonbox

	iwidgets::buttonbox $buttonbox -padx 0 -pady 2

	$buttonbox add OK -text "Create Scan" -command \
		[ itcl::code mxgui_list_create_scan $window $record_list \
		$scan_name $scan_record $scan_type $old_record_exists ]

	$buttonbox add Cancel -text Cancel -command \
		[ itcl::code destroy $window ]

	$buttonbox add Run -text "Create and Run Scan" -command \
	    [ itcl::code mxgui_list_create_and_run_scan $window $record_list \
		$scan_name $scan_record $scan_type $old_record_exists ]

	$buttonbox default OK

	pack $buttonbox -padx 10 -pady 10

#
# If this is an old scan, make the contents of the form match the
# parameters of the old scan.
#

	if { $old_record_exists != 0 } {
		mxgui_list_scan_copy_motor_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_input_device_fields_from_record \
				$window $record_list $scan_record

		mxgui_scan_copy_measurement_fields_from_record \
				$window $record_list $scan_record \
				$show_measurement_parameters

		mxgui_scan_copy_datafile_and_plot_fields_from_record \
				$window $record_list $scan_record
	}

#
# Make the window appear on the screen.
#
	wm deiconify $window

	update
}

####################################################################
#                                                                  #
#  The following procedure does the initial setup of the input     #
#  scan fields on the GUI scan setup screen.                       #
#                                                                  #
####################################################################

proc mxgui_list_initialize_input_scan_steps_display { window \
			record_list scan_record old_record_exists } {

	set scan_steps_frame $window.scan_steps_frame

	frame $scan_steps_frame
	pack $scan_steps_frame -padx 5 -pady 5

	iwidgets::entryfield $scan_steps_frame.num_scan_steps \
				-labeltext "Number of input scan steps:"

	if { $old_record_exists == 0 } {
		set num_scan_steps 1
	} else {
		set num_scan_steps \
			[ $scan_record get_field num_measurements ]
	}

	$scan_steps_frame.num_scan_steps clear
	$scan_steps_frame.num_scan_steps insert 0 $num_scan_steps

	$scan_steps_frame.num_scan_steps configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $scan_steps_frame.num_scan_steps
}

####################################################################
#                                                                  #
#  The following procedure does the initial setup of the motor     #
#  fields on the GUI scan setup screen.                            #
#                                                                  #
####################################################################

proc mxgui_list_initialize_motor_display { window record_list scan_record \
						old_record_exists } {
#
# Display the list of motors.
#

	set num_motors_frame $window.num_motors_frame
	set motor_frame      $window.motor_frame

	frame $num_motors_frame
	pack $num_motors_frame -padx 5 -pady 5

	if { $old_record_exists == 0 } {
		set num_motors 1
	} else {
		set num_motors [ $scan_record get_field num_motors ]
	}

	iwidgets::entryfield $num_motors_frame.num_motors \
				-labeltext "Number of motors:" -width 8

	$num_motors_frame.num_motors clear
	$num_motors_frame.num_motors insert 0 $num_motors

	$num_motors_frame.num_motors configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_motors_frame.num_motors -side left

#
# old_num_motors is meant to be a hidden field that stores the
# old number of motors in a place the other procedures can find it.
# So don't make it visible by packing it or gridding it, etc.
#
	label $num_motors_frame.old_num_motors -text 0

# Create the frame for the list of motors.

	frame $motor_frame
	pack $motor_frame -padx 5 -pady 5

# Put labels at the top of the motor list.

	label $motor_frame.name -text "Motor name"

	grid $motor_frame.name -row 0 -column 0

	label $motor_frame.spacer -text "              "

	grid $motor_frame.spacer -row 0 -column 1

	label $motor_frame.position_filename_header \
			-text "Requested Motor Positions Filename"

	grid $motor_frame.position_filename_header -row 0 -column 2

# Set up the entry field for the motor positions filename

	iwidgets::entryfield $motor_frame.position_filename -width 40

	$motor_frame.position_filename clear

	grid $motor_frame.position_filename -row 1 -column 2

# Set up the individual motor information.

	$num_motors_frame.num_motors configure -command [ itcl::code \
		mxgui_list_scan_redo_motor_fields \
			$motor_frame $num_motors_frame \
			$record_list $scan_record $old_record_exists ]

# Initialize the motor display.

	mxgui_list_scan_redo_motor_fields $motor_frame $num_motors_frame \
			$record_list $scan_record $old_record_exists

}

####################################################################
#                                                                  #
#  The following routine causes the motor fields to be redisplayed #
#  after any change in the number of motors for the scan.          #
#                                                                  #
####################################################################

proc mxgui_list_scan_redo_motor_fields { motor_frame num_motors_frame \
			record_list scan_record old_record_exists } {

	set mywidth 12

# Get the old and new number of motors.

	set old_num_motors [ $num_motors_frame.old_num_motors cget -text ]

	set num_motors [ $num_motors_frame.num_motors get ]

	if { $num_motors > $old_num_motors } {

		# Add new motors to the list of motors.

		set row [ expr ( $old_num_motors + 1 ) ]

		for { set i $old_num_motors } { $i < $num_motors } { incr i } {

		    set thismotor $motor_frame.motor$i

		    iwidgets::entryfield ${thismotor}_name -width $mywidth \
			-command [ itcl::code mxgui_list_change_motor \
				$record_list $thismotor ]

		    grid ${thismotor}_name -row $row -column 0

		    incr row
		}
	} else {

		# Delete old motors from the list of motors.

		for { set i $num_motors } { $i < $old_num_motors } { incr i } {

		    set thismotor $motor_frame.motor$i

		    destroy ${thismotor}_name
		}
	}

	$num_motors_frame.old_num_motors configure -text $num_motors
}

####################################################################
#                                                                  #
#  The following routine causes the motor scan fields to be        #
#  initialized to the values from an old scan record.              #
#                                                                  #
####################################################################

proc mxgui_list_scan_copy_motor_fields_from_record { window \
						record_list scan_record } {

	set num_motors    [ $scan_record get_field num_motors ]
	set motor_list    [ split [ $scan_record get_field motor_array ] ]

	set motor_frame $window.motor_frame

	for { set i 0 } { $i < $num_motors } { incr i } {

		set thismotor $motor_frame.motor$i

		set motor_name [ lindex $motor_list $i ]

		${thismotor}_name     insert end $motor_name
	}

	set position_filename \
		[ $scan_record get_field position_filename ]

	$motor_frame.position_filename	insert end $position_filename
}

####################################################################
#                                                                  #
# This procedure initializes the contents of the motor scan fields #
# when a new motor name is typed in.                               #
#                                                                  #
####################################################################

proc mxgui_list_change_motor { record_list thismotor } {

# Get the name of the motor.

	set motor_name [ ${thismotor}_name get ]

	if [ catch { mxgui_list_get_motor_record $record_list $motor_name } \
		motor_record ] {

		mxgui_list_clear_motor_fields $thismotor
		return
	}

	return
}

####################################################################
#                                                                  #
# This procedure gets the specified motor record and checks for    #
# various error conditions.                                        #
#                                                                  #
####################################################################

proc mxgui_list_get_motor_record { record_list motor_name } {

# Does a motor by this name exist?

	if [ catch { $record_list get_record $motor_name } motor_record ] {

		Mx::error_message "No Such Motor" \
			"The motor '$motor_name' does not exist." "" ""

		return -code error ""
	}

# Is this record really a motor?

	set superclass [ $motor_record get_field mx_superclass ]
	set class [ $motor_record get_field mx_class ]

	if { $superclass != "device" } {
		# Make the error message grammatically correct.

		if { $superclass == "interface" } {
			set a_word "an"
		} else {
			set a_word "a"
		}

		set t "Record '$motor_name' is not a motor record.  "
		append t \
		"Instead it is $a_word $superclass record of class '$class'."

		Mx::error_message "Illegal Record Type" $t "" ""

		return -code error ""
	}
	if { $class != "motor" } {
		set t "Record '$motor_name' is not a motor record.  "
		append t "Instead it is a device record of class '$class'."

		Mx::error_message "Illegal Record Type" $t "" ""

		return -code error ""
	}
	return $motor_record
}


####################################################################
#                                                                  #
# This procedure clears all the fields for a given motor           #
#                                                                  #
####################################################################

proc mxgui_list_clear_motor_fields { thismotor } {
	${thismotor}_name clear
}

####################################################################
#                                                                  #
# The following routine sets up the description for the new scan.  #
#                                                                  #
####################################################################

proc mxgui_list_create_scan { window record_list scan_name scan_record \
				scan_type old_record_exists } {

	set description "$scan_name  scan linear_scan $scan_type \"\" \"\" "

# Append the number of times to run this scan.

	set num_scans [ $window.num_scans_frame.num_scans get ]

	append description "$num_scans "

# Append the num_independent_variables and num_motors fields.

	switch -- $scan_type {
		input_scan  {
			set num_independent_variables 1
			set num_motors 0
		}
		2theta_scan {
			set num_independent_variables 1
			set num_motors 2
		}
		default {
			set num_motors \
				[ $window.num_motors_frame.num_motors get ]

			set num_independent_variables $num_motors
		}
	}

	if { $num_motors < 0 } {
		Mx::error_message "Illegal Number of Motors" \
	"The number of motors ($num_motors) may not be less than zero." "" ""

		return
	}
	if { $num_motors == 0 } {
		if { $scan_type != "input_scan" } {
			Mx::error_message "Illegal Number of Motors" \
	"The number of motors may not be zero for scan type '$scan_type'." "" ""

			return
		}
	}

	append description "$num_independent_variables $num_motors "

# Append the motor names, if any.

	for { set i 0 } { $i < $num_motors } { incr i } {

		set motor_name [ $window.motor_frame.motor${i}_name get ]

		set name_length [ string length [ string trim $motor_name ] ]

		if { $name_length == 0 } {
			Mx::error_message "Empty Motor Name" \
			"No motor name was specified for motor $i." "" ""

			return
		}
		append description "$motor_name "
	}

# Append the input devices, if any.

	set num_inputs_frame    $window.readout_frame.inputs.num_inputs_frame
	set input_device_frame  $window.readout_frame.inputs.input_device_frame

	set num_input_devices [ $num_inputs_frame.num_input_devices get ]

	if { $num_input_devices < 0 } {
		Mx::error_message "Illegal number of input devices" \
"The number of input devices ($num_input_devices) may not be less than zero." \
			"" ""

		return
	}
	append description "$num_input_devices "

	for { set i 0 } { $i < $num_input_devices } { incr i } {

		set input_device_name \
			[ $input_device_frame.device${i}_name get ]

		set name_length \
			[ string length [ string trim $input_device_name ] ]

		if { $name_length == 0 } {
			Mx::error_message "Empty input device name" \
		"No input device name was specified for input device $i." "" ""

			return
		}
		append description "$input_device_name "
	}

# Append the after scan action ( currently always 0 ).

	append description "0 "

# Append the settling time.

	set measurement_frame $window.readout_frame.measurement

	set settling_time [ $measurement_frame.sframe.settling_time get ]

	set length [ string length [ string trim $settling_time ] ]

	if { $length == 0 } {
		Mx::error_message "Empty settling time" \
			"No settling time was specified." "" ""

		return
	}

	append description "$settling_time "

# Append the measurement type.

	set measurement_type \
	    [ mxgui_scan_get_measurement_type_from_gui $measurement_frame ]

	append description "$measurement_type "

# Append the measurement information.

	switch -exact -- $measurement_type {
		preset_time { set measurement_args \
	[ mxgui_scan_get_preset_time_args $measurement_frame.arguments_frame ]}

		preset_count { set measurement_args \
	[ mxgui_scan_get_preset_count_args $measurement_frame.arguments_frame ]}

		default { set measurement_args \
	[ mxgui_scan_get_default_args $measurement_frame.arguments_frame ]}
	}

	if { $measurement_args == "" } {
		return
	}

	append description "$measurement_args "

# Append the datafile information.

	set datafile_frame $window.datafile_plot_frame.datafile_frame

	append description "[ string tolower [ $datafile_frame.type get ] ] "

	append description "[ $datafile_frame.name get ] "

# Append the plot information.

	set plot_frame $window.datafile_plot_frame.plotfile_frame

	append description "[ string tolower [ $plot_frame.type get ] ] "

	append description "\"[ $plot_frame.arguments get ]\" "

# Append the file containing the requested motor positions.

	append description "\"[ $window.motor_frame.position_filename get ]\""

# Create the new scan record.

	if { $old_record_exists != 0 } {
		$scan_record delete_record
	}

	if [ catch { $record_list create_record_from_description \
					$description } new_record ] {

		set t "Attempt to create scan '$scan_name' failed."

		Mx::error_message "Record Creation Failed" $t "" ""

		return
	}

	if [ catch { $new_record finish_record_initialization } ] {

		set t "Attempt to initialize scan record '$scan_name' failed."

		Mx::error_message "Record Initialization Failed" $t "" ""

		return
	}

# Get rid of the scan setup window

	wm withdraw $window
	destroy $window

# Remember that we now have unsaved scans.

	$record_list configure -unsaved_scans 1
}

proc mxgui_scan_get_preset_time_args { arguments_frame } {

	set measurement_time [ $arguments_frame.measurement_time get ]

	set length [ string length [ string trim $measurement_time ] ]

	if { $length == 0 } {
		Mx::error_message "Empty measurement time" \
			"No measurement time was specified." "" ""

		return ""
	}

	set timer_name [ $arguments_frame.timer_name get ]

	return "\"$measurement_time $timer_name\""
}

proc mxgui_scan_get_preset_count_args { arguments_frame } {

	set measurement_counts [ $arguments_frame.measurement_counts get ]

	if { $length == 0 } {
		Mx::error_message "Empty measurement counts" \
			"No measurement counts was specified." "" ""

		return ""
	}

	set scaler_name [ $arguments_frame.scaler_name get ]

	return "\"$measurement_counts $scaler_name\""
}

proc mxgui_scan_get_default_args { arguments_frame } {

	set measurement_arguments [ $arguments_frame.measurement_arguments get ]

	return "\"$measurement_arguments\""
}

####################################################################
#                                                                  #
# The following routine calls mxgui_list_create_scan to create a  #
# new scan record and then runs the scan.                          #
#                                                                  #
####################################################################

proc mxgui_list_create_and_run_scan { window record_list \
				scan_name scan_record \
				scan_type old_record_exists } {

	mxgui_list_create_scan $window $record_list $scan_name \
		$scan_record $scan_type $old_record_exists

	#
	#  Check to see if the files created by this scan will
	#  overwrite already existing files.
	#

	set status [ mxgui_check_for_datafile_name_collision $scan_record ]

	#
	#  If the returned status is 0, the user has decided to
	#  abort the scan since preexisting files would be
	#  overwritten.
	#

	if { $status == 0 } {
		return
	}

	#
	#  Prompt the user for scan header changes
	#

	set datafile_type [ get_datafile_type $scan_record ]

	set status [ menubar_setup_scan_header_change_dialog \
				$record_list $datafile_type $scan_record ]

	#
	#  If the returned status is 1, we have to do the scan ourself.
	#  Otherwise, a callback that was set up in the procedure
	#  menubar_setup_scan_header_change_dialog will do it for us.
	#

	if { $status == 1 } {
		mxgui_perform_scan $scan_record
	}
}

